package Amethyst::Brain::Infobot::Module::Google;

use strict;
use vars qw(@ISA);
use Data::Dumper;
use POE;
use POE::Component::Client::HTTP;
use HTTP::Request::Common;
use HTTP::Response;
use WWW::Search;
use WWW::SearchResult;
use Amethyst::Message;
use Amethyst::Brain::Infobot;
use Amethyst::Brain::Infobot::Module;

@ISA = qw(Amethyst::Brain::Infobot::Module);

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(
					Name		=> 'Google',
					Regex		=> qr/^(?:google|search)\s+for\s+(.*)$/i,
					Usage		=> '(google|search) for .*',
					Description	=> "Search Google",
					@_
						);

	return bless $self, $class;
}

sub init {
	my $self = shift;

	eval {
		spawn POE::Component::Client::HTTP(
						# Agent	=> 'Lynx/2.8rel.1 libwww-FM/2.14',
						Alias	=> 'googleua',
						Timeout	=> 30,
							);
	};
	if ($@) {
		die $@ unless $@ =~ /^alias is in use by another session/;
	}
}

sub action {
    my ($self, $message, $query) = @_;

	my %states = map { $_ => "handler_$_" } qw(
					_start response
						);

	print STDERR "Creating child session for google\n";

	POE::Session->create(
		package_states	=> [ ref($self) => \%states ],
		args			=> [ $self, $message, $query ],
			);

	return 1;
}


sub parse_response {
	my ($self, $response) = @_;

	$self->{_debug} = 10;

	# print STDERR "Self is $self\n";

	local *LOGFILE;
	open(LOGFILE, ">google.log") or die "Can't open file: $!";
	print LOGFILE $response->content;
	print LOGFILE "\n\n\n";

	# parse the output
	my ($HEADER, $HITS, $TRAILER, $POST_NEXT) = (1..10);

	my @hits = ();
	my $hit = undef;

	my $approx_count = -1;

	my $state = $HEADER;

	my $content = $response->content;
	$content =~ s/^.*?Searched the web for//;
	# my @lines = split(/\n/, $response->content);
	my @lines = split(/(?=<p class=)/, $response->content);

	foreach (@lines) {
		next unless /\S/; # short circuit for blank lines

		# print LOGFILE substr($_, 0, 70) . "\n" if $self->{_debug};
		print LOGFILE $_ . "\n" if $self->{_debug};

		if ($state == $HEADER && m/about <b>([\d,]+)<\/b>/) {
			$approx_count = $1;
			print LOGFILE "Found Total: $approx_count\n" ;
			$state = $HITS;
		}
		elsif ($state == $HITS &&
			m|<p class=g><a href=[^\s]*(http[^&>]*)[^>]*>(.*?)</a>|i) {
			my ($url, $title) = ($1, $2);
			$hit = new WWW::SearchResult();
			push(@hits, $hit);

			print LOGFILE "**Found HIT Line**\n" if ($self->{_debug});
			$url =~ s/(>.*)//g;
			$hit->add_url(WWW::Search::strip_tags($url));
			$title = "No Title" if ($title =~ /^\s+/);

			$hit->title(WWW::Search::strip_tags($title));
			$state = $HITS;
		} 
		elsif ($state == $HITS && m|Description:</font></span>\s*(.*)<br>|i) {
			print LOGFILE "**Parsing Description Line**\n" if ($self->{_debug});
			if ($hit) {
				my $desc = $1;
				$desc =~ s/<.*?>//g;
				$desc =~ s/Category.*//;
				$hit->description($desc);
				$state = $HITS;
			}
			else {
				print LOGFILE "ERROR: No hit when parsing description\n";
			}
		} 
		elsif ($state == $HITS && m@<form method=GET@i) {
			print LOGFILE "**Found Last Line**\n" if ($self->{_debug});
			# end of hits
			$state = $TRAILER;
		}
		else {
			print LOGFILE "**No match**\n" if ($self->{_debug});
		}
	}

	print LOGFILE "\n\n\n";
	close(LOGFILE);

	return @hits;
}

sub handler_response {
	my ($kernel, $heap, $session, $rqdata, $rsdata) =
					@_[KERNEL, HEAP, SESSION, ARG0, ARG1];
	my ($request) = $rqdata->[0];
	my ($response) = $rsdata->[0];

	unless ($response->is_success) {
		my $reply = $heap->{Module}->reply_to($heap->{Message},
						"HTTP Request failed");
		$reply->send;
		print STDERR $response->error_as_HTML;
		return;
	}

	# What we want here is a WWW::Search with a _separate_ parser
	# for pages which we have alrady retrieved.

	my @hits = parse_response($heap->{Module}, $response);

	my $module = $heap->{Module};

	# print STDERR Dumper(\@hits);

	if (@hits) {
		@hits = @hits[0..3] if @hits > 4;
		foreach my $hit (@hits) {
			my $url = $hit->url;
			my $title = $hit->title;
			my $description = $hit->description || '';
			my $reply = $module->reply_to($heap->{Message},
							$url . ": " . $title);
			$reply->send;
		}
	}
	else {
		my $reply = $module->reply_to($heap->{Message}, 'No results');
		$reply->send;
	}
}

sub handler__start {
	my ($kernel, $heap, $session, $module, $message, $query) =
					@_[KERNEL, HEAP, SESSION, ARG0, ARG1, ARG2];

	$heap->{Module} = $module;
	$heap->{Message} = $message;
	$heap->{Query} = $query;

	my $uri = new URI('http://www.google.com:80/search');
	$uri->query_form(q => $query);
	$uri = $uri->canonical;
	print STDERR "Searching for " . $uri->as_string . "\n";
	my $request = new HTTP::Request(GET => $uri);
	# my $postback = $session->postback('response');

	$kernel->post('googleua', 'request', 'response', $request);
}

1;
